@@ -14,7 +14,9 @@
 #    under the License.
 
 import copy
+import requests
 import StringIO
+import testtools
 
 from glanceclient.common import http
 
@@ -31,8 +33,9 @@ class FakeAPI(object):
 
     def raw_request(self, *args, **kwargs):
         fixture = self._request(*args, **kwargs)
-        body_iter = http.ResponseBodyIterator(StringIO.StringIO(fixture[1]))
-        return FakeResponse(fixture[0]), body_iter
+        resp = FakeResponse(fixture[0], StringIO.StringIO(fixture[1]))
+        body_iter = http.ResponseBodyIterator(resp)
+        return resp, body_iter
 
     def json_request(self, *args, **kwargs):
         fixture = self._request(*args, **kwargs)
@@ -63,3 +66,47 @@ class FakeResponse(object):
 
     def read(self, amt):
         return self.body.read(amt)
+
+
+class TestCase(testtools.TestCase):
+    TEST_REQUEST_BASE = {
+        'config': {'danger_mode': False},
+        'verify': True}
+
+
+class TestResponse(requests.Response):
+    """
+    Class used to wrap requests.Response and provide some
+    convenience to initialize with a dict
+    """
+    def __init__(self, data):
+        self._text = None
+        super(TestResponse, self)
+        if isinstance(data, dict):
+            self.status_code = data.get('status_code', None)
+            self.headers = data.get('headers', None)
+            # Fake the text attribute to streamline Response creation
+            self._text = data.get('text', None)
+        else:
+            self.status_code = data
+
+    def __eq__(self, other):
+        return self.__dict__ == other.__dict__
+
+    @property
+    def text(self):
+        return self._text
+
+
+class FakeTTYStdout(StringIO.StringIO):
+    """A Fake stdout that try to emulate a TTY device as much as possible."""
+
+    def isatty(self):
+        return True
+
+    def write(self, data):
+        # When a CR (carriage return) is found reset file.
+        if data.startswith('\r'):
+            self.seek(0)
+            data = data[1:]
+        return StringIO.StringIO.write(self, data)
